summaryrefslogtreecommitdiff
path: root/source3/include/dlinklist.h
diff options
context:
space:
mode:
Diffstat (limited to 'source3/include/dlinklist.h')
-rw-r--r--source3/include/dlinklist.h18
1 files changed, 12 insertions, 6 deletions
diff --git a/source3/include/dlinklist.h b/source3/include/dlinklist.h
index d510aad028..c35155d9bc 100644
--- a/source3/include/dlinklist.h
+++ b/source3/include/dlinklist.h
@@ -37,17 +37,17 @@
}\
}
-
-/* remove an element from a list */
+/* remove an element from a list - element doesn't have to be in list. */
#define DLIST_REMOVE(list, p) \
{ \
if ((p) == (list)) { \
(list) = (p)->next; \
if (list) (list)->prev = NULL; \
} else { \
- (p)->prev->next = (p)->next; \
+ if ((p)->prev) (p)->prev->next = (p)->next; \
if ((p)->next) (p)->next->prev = (p)->prev; \
} \
+ (p)->next = (p)->prev = NULL; \
}
/* promote an element to the top of the list */
@@ -57,10 +57,9 @@
DLIST_ADD(list, p) \
}
/* demote an element to the top of the list, needs a tmp pointer */
#define DLIST_DEMOTE(list, p, tmp) \
{ \
- DLIST_REMOVE(list, p) \
if (!(list)) { \
(list) = (p); \
(p)->next = (p)->prev = NULL; \
@@ -71,3 +70,10 @@
(p)->prev = (tmp); \
} \
}
+
+/* demote an element to the top of the list, needs a tmp pointer */
+#define DLIST_DEMOTE(list, p, tmp) \
+{ \
+ DLIST_REMOVE(list, p) \
+ DLIST_ADD_END(list, p, tmp) \
+}
@@ -57,10 +57,9 @@
DLIST_ADD(list, p) \
}
-/* demote an element to the top of the list, needs a tmp pointer */
-#define DLIST_DEMOTE(list, p, tmp) \
+/* hook into the end of the list - needs a tmp pointer */
+#define DLIST_ADD_END(list, p, tmp) \
{ \